@@ -303,6 +303,15 @@ static int erofs_mkfs_feat_set_dedupe(bool en, const char *val,
 	return 0;
 }
 
+static int erofs_mkfs_feat_set_fragdedupe(bool en, const char *val,
+					  unsigned int vallen)
+{
+	if (vallen)
+		return -EINVAL;
+	cfg.c_nofragdedupe = !en;
+	return 0;
+}
+
 static struct {
 	char *feat;
 	int (*set)(bool en, const char *val, unsigned int len);
@@ -312,6 +321,7 @@ static struct {
 	{"fragments", erofs_mkfs_feat_set_fragments},
 	{"all-fragments", erofs_mkfs_feat_set_all_fragments},
 	{"dedupe", erofs_mkfs_feat_set_dedupe},
+	{"fragdedupe", erofs_mkfs_feat_set_fragdedupe},
 	{NULL, NULL},
 };
 
@@ -1429,7 +1439,8 @@ int main(int argc, char **argv)
 	if (erofstar.index_mode && g_sbi.extra_devices && !erofstar.mapfile)
 		g_sbi.devs[0].blocks = BLK_ROUND_UP(&g_sbi, erofstar.offset);
 
-	if (erofs_sb_has_fragments(&g_sbi)) {
+	if ((cfg.c_fragments || cfg.c_extra_ea_name_prefixes) &&
+	    erofs_sb_has_fragments(&g_sbi)) {
 		erofs_update_progressinfo("Handling packed data ...");
 		err = erofs_flush_packed_inode(&g_sbi);
 		if (err)
